BeginPackage["digraph`", {"Graphics`Arrow`"}]

computegraph::usage =
	"computegraph [ matrix ] generates a directed graph representation
	 of a Markov Chain which is input as a transition probability 
	 matrix.  Cycles in the graph are represented as 'double nodes'."

computenetwork::usage =
	"computenetwork [matrix,vector] generates a diagram of a Jackson
	network which was given as input.  Matrix represents the connectivity
	matrix for the nodes of the Jackson network.  Vector represents a
	list of inputs to the network."
	
Begin["`Private`"]

computegraph [ matrix_List ] := Module [ {},
	
	Show [ Graphics [ generategraph[ matrix ] ], PlotRange -> All ]

]

computenetwork [ matrix_List, vector_List ] := Module [ {network},

	network = generategraph [ matrix ];
	network = Append [ network, generateinputs [ vector ] ];

	Show [ Graphics [ network ], PlotRange -> All ]

]

generateinputs [ vector_List ] := Module [ {vectorsize, vectorlist={} },

	vectorsize = Length[ vector ];

	Do [ 
		If [ SameQ [ vector [[i]], 0 ],
		     {},
		     vectorlist = Append [vectorlist, findarrow [ i, vector[[i]], vectorsize ] ]
		],
		{i,1,vectorsize}
	];
	Return [ vectorlist ]
]

findarrow [ dot_, label_, size_ ] := Module [ {point1, point2, theta},

	point1 = cordinates [ dot, size, size ];
	point2 = cordinates [ dot, size, 1.75 size ];

	theta = ( dot 2 Pi / size ) + Pi/1.1;

	point1[[1]] += N[ 0.5 Cos [ theta ]];
	point1[[2]] += N[ 0.5 Sin [ theta ]];

	Return [ {Arrow[point2, point1], 
		  Text[ label, newcord [ point1, point2 ] ] } 
	]
]

generategraph [ transprobability_List ] := Module [ {ll,j, mygraph = {}, matrixsize },

	matrixsize = Length [ transprobability ];

	Do [ 
		If [ SameQ [ transprobability [[ll,j]] , 0 ] || SameQ [ transprobability [[ll,j]], 0. ],
		     {},
		     If [ ll == j,
	                  mygraph = Append[ mygraph, 
				            drawcircle [ ll, transprobability[[ll,j]], 
				            matrixsize ] ],
				     
                          mygraph = Append[ mygraph, 
	                                    drawline [ ll, j, transprobability[[ll,j]],
				            matrixsize ] ]
		     ],
		],
		{ ll, 1, matrixsize }, {j, 1, matrixsize }
	];
	Return[ mygraph ]
]
	
drawcircle [ i_, label_, matrixsize_ ] := Module [ {theta, point1, temp1, lcv},

	point1 = cordinates[i, matrixsize, matrixsize];
	temp1 = Range [ 2 ];

	theta = (( i 2 Pi ) / matrixsize) + Pi/1.1;

	temp1[[1]] = 1.5 matrixsize Cos[theta];
	temp1[[2]] = 1.5 matrixsize Sin[theta];
	
	Return [ { Circle[point1,0.4 ], Text [ label, temp1 ] }
	]	
]
	
drawline[ i_, j_, label_, matrixsize_ ] := Module [ {point1, point2, theta, cor1, cor2, lcv },

  point1 = cordinates[i, matrixsize, matrixsize ];
  point2 = cordinates[j, matrixsize, matrixsize ];

  theta = ArcTan [ (point2 [[2]] - point1[[2]]) / (point2[[1]] - point1[[1]]) ];
  
  cor1 = Range [ 2 ];
  cor2 = Range [ 2 ];

  If [ point1[[1]] < point2[[1]],
         {
  	 cor1[[1]] = point1[[1]] + 0.5 Abs[Cos [ theta ]];
  	 cor2[[1]] = point2[[1]] - 0.5 Abs[Cos [ theta ]];
  	 },
  	 {
   	 cor1[[1]] = point1[[1]] - 0.5 Abs[Cos [ theta ]];
  	 cor2[[1]] = point2[[1]] + 0.5 Abs[Cos [ theta ]];
 	 }
  ];
   
  If [ point1[[2]] < point2[[2]],
     	{
  	cor1[[2]] = point1[[2]] + 0.5 Abs[Sin [ theta ]];
  	cor2[[2]] = point2[[2]] - 0.5 Abs[Sin [ theta ]];
  	},
  	{
   	cor1[[2]] = point1[[2]] - 0.5 Abs[Sin [ theta ]];
  	cor2[[2]] = point2[[2]] + 0.5 Abs[Sin [ theta ]];
 	}
  ];
      	
  Return [ { Circle[ point1, 0.5 ], Text[ i-1, point1 ],
	     Circle[ point2, 0.5 ], Text[ j-1, point2 ],		 
	     Arrow[ cor1, cor2 ], Text[ label, newcord [ point1, point2 ] ] }
  ]
]			 

newcord [ pnt1_, pnt2_ ] := Module [ {},


	If [ pnt1[[1]] < pnt2[[1]],	
		Return [ { ((pnt1[[1]] + pnt2[[1]])/2)+0.5, ((pnt1[[2]] + pnt2[[2]])/2)+0.5 } ],
		Return [ { ((pnt1[[1]] + pnt2[[1]])/2)-0.5, ((pnt1[[2]] + pnt2[[2]])/2)-0.5 } ]
        ];
]

cordinates [ x_, matrixsize_, diameter_ ] := Module [ {theta},

	theta = ( x 2 Pi / matrixsize ) + Pi/1.1;
	Return [ { N[diameter * Cos [ theta ]], 
	 	   N[diameter * Sin [ theta ]] }
	]
]

End [ ]

EndPackage [ ]





